==> resh-5.1.0.9-py3-none-any.whl: reface/line/scanner/scanner.py <==
import json
import logging
import os
import threading

from asgiref.sync import async_to_sync

from reface.line.worker import Worker
from resh.domains import dali1, dali2
from resh.utils import const_values as v
from resh.utils.const import INSTANCE
from resh.utils.errors import DeviceNotAnswered
from resh.utils.settings import conf


class Scanner(Worker):
    folder = 'scan'
    marks = {}

    def file(self, domain, address_int): return os.path.join(self.root, domain, str(address_int))

    def _run_thread(self, data):
        addresses = data['addresses']
        keys = data['keys']

        if v.FADE_TIME in keys.get(dali1.Dali1Protocol.domain_name, []):
            keys[dali1.Dali1Protocol.domain_name].remove(v.FADE_TIME)
            if v.FADE_TIME_AND_RATE not in keys:
                keys[dali1.Dali1Protocol.domain_name].append(v.FADE_TIME_AND_RATE)

        if v.FADE_RATE in keys.get(dali1.Dali1Protocol.domain_name, []):
            keys[dali1.Dali1Protocol.domain_name].remove(v.FADE_RATE)
            if v.FADE_TIME_AND_RATE not in keys:
                keys[dali1.Dali1Protocol.domain_name].append(v.FADE_TIME_AND_RATE)

        d1 = addresses.get(v.DALI, [])
        d2 = addresses.get(v.DALI2, [])
        keys_1 = [v.TYPES]
        keys_2 = []
        for k in keys.get(v.COMMON, []):
            if k in dali1.dump.queries:
                keys_1.append(k)
            if k in dali2.dump.queries_device():
                keys_2.append(k)

        keys_1 += keys.get(dali1.Dali1Protocol.domain_name, []) + keys.get(v.SWITCH, []) + keys.get(v.COLOUR, [])
        keys_2 += keys.get(dali2.Dali2Protocol.domain_name, []) + keys.get(v.SENSORS, []) + keys.get(v.BUTTONS, [])

        total_count = len(d1) + len(d2)
        progress = 0

        self.send_progress(1)

        def on_step(*args, **kwargs):
            self.on_step(cmd.domain_name, *args, **kwargs)

        for a in sorted(set(d1+d2), key=lambda a: int(a[1:])):
            if self.stopped:
                break
            for d, keys, dump in ((d1, keys_1, dali1.dump), (d2, keys_2, dali2.dump)):
                if self.stopped:
                    break
                if a in d:
                    cmd = dump(self.module, self.channel, a)
                    conf.interface = self.box
                    cmd.set_step_handler(on_step)
                    cmd.custom_stack = keys
                    try:
                        cmd.exec()
                    except DeviceNotAnswered:
                        self.clear_device(cmd.domain_name, a)
                    except BaseException as ex:
                        print(str(ex))
                    progress += 1
                    self.send_progress(min(99, progress * 100 // total_count))

        self.send_progress(100)

    def stop(self):
        if self._command:
            self._command.set_break(None, None)
        self.send_progress(0)

    def on_step(self, dali_type,  address, key, value, index=None):
        if isinstance(value, DeviceNotAnswered):
            value = 'unanswered'
        elif isinstance(value, DeviceNotAnswered):
            value = 'can error'
        elif isinstance(value, BaseException):
            value = 'error'
        self.update_value(dali_type, address, key, value, index)

    def update_value(self, domain, address, key, value, index=None):
        data = self.read(domain, address, default={})

        if index is not None:
            data.setdefault('instances', [])
            found = False
            for item in data['instances']:
                if item['index'] == index:
                    item[key] = value
                    found = True
                    break
            if not found:
                data['instances'].append({'index': index, key: value})
        else:
            if key == v.SCENES and data.get(key, None):
                data[key].update(value)
                value = data[key]
            else:
                data[key] = value

        self.write(domain, address, value=data)

        async_to_sync(self.layer.group_send)(
            self.group,
            {
                'type': 'chat_message',
                'message': {
                    'address': address,
                    'type': domain,
                    'index': index,
                    'data': {
                        key: value,
                    }
                }
            }
        )

    def clear_device(self, domain, address):
        """ Удаляет файл девайса и шлёт в сокет структуру для удаления

        :param domain: dali or dali2
        :param address: device address like "D0..D63" or "0..63"
        :return:
        """
        self.remove(domain, address[1:])

        async_to_sync(self.layer.group_send)(
            self.group,
            {
                'type': 'chat_message',
                'message': {
                    'remove': address,
                    'type': domain,
                }
            }
        )

    def update_device(self, domain, address):
        async_to_sync(self.layer.group_send)(
            self.group,
            {
                'type': 'chat_message',
                'message': {
                    'status': {
                        'address': address,
                        'type': domain,
                        'device': self.read(domain, address[1:], default={})
                    }
                }
            }
        )

    def set_value(self, action, data):
        type = data['type']
        key = data['key']
        value = data.get('value', '')
        args = [self.module, self.channel, data['address']]
        command = None

        if type == dali1.dump.domain_name:
            if action == 'get' and key in (v.FADE_TIME, v.FADE_RATE):
                command = dali1.query_fade_time_and_rate
            else:
                command = dali1.dump.queries.get(key, {}).get(action)
                if command is None:
                    for commands_dict in dali1.dump.extended.values():
                        command = commands_dict.get(key, {}).get(action)
                        if command:
                            break
        elif type == dali2.dump.domain_name:
            command = dali2.dump.queries_device().get(key, {}).get(action)
            if command is None:
                args.append(data['instance'])
                command = dali2.dump.queries_instance().get(key, {}).get(action)
                if command is None:
                    if action == 'set' and key == v.IS_ENABLED:
                        bool_value = json.loads(value)
                        if bool_value:
                            command = dali2.enable_instance
                        else:
                            command = dali2.disable_instance
                    else:
                        for commands_dict in dali2.dump.queries_instance_type().values():
                            command = commands_dict.get(key, {}).get(action)
                            if command:
                                break

        conf.interface = self.box

        if command in (dali1.discovery, dali1.identify, dali2.identify):
            try:
                self.marks.get(command.domain_name, {}).get(key, {}).pop(data['address']).stop()
            except:
                pass

            if value == 'true':
                def __exec(inst):
                    _command = command(*args)
                    inst.marks.setdefault(command.domain_name, {})
                    inst.marks[command.domain_name].setdefault(key, {})
                    inst.marks[command.domain_name][key][data['address']] = command(*args)
                    inst.marks[command.domain_name][key][data['address']].exec()
                    inst.update_value(data['type'], data['address'][1:], key, False)

                if data['address'].startswith('D'):
                    self.update_value(data['type'], data['address'][1:], key, True)
                threading.Thread(target=__exec, args=[self], name='qwe', daemon=False).start()
            else:
                self.update_value(data['type'], data['address'][1:], key, False)

        elif command:
            if command.opt_params:
                args.append(data.get('value', ''))
            command = command(*args)
            res = command.exec()
            if res is not None:
                r_kwargs = {}
                if INSTANCE in command.def_params:
                    r_kwargs['index'] = command.instance._value
                self.update_value(type, command.address.address, key, res, **r_kwargs)

    def call_command(self, data):
        domain = dali1.Dali1Protocol if data['type'] == dali1.Dali1Protocol.domain_name else dali2.Dali2Protocol

        try:
            args = (self.module, self.channel, data['address'], *data['params'])
            method = domain.from_name(data['command'])
            conf.interface = self.box
            method(*args).exec()

            if method in (dali1.set_short_address, dali2.set_short_address,
                          dali1.swap_short_addresses, dali2.swap_short_addresses,
                          dali1.remove_short_address, dali2.remove_short_address):
                if method in (dali1.remove_short_address, dali2.remove_short_address):
                    self.clear_device(data['type'], data['address'])
                else:
                    f1 = self.file(data['type'], data['address'][1:])
                    f2 = self.file(data['type'], data['params'][0][1:])
                    if method in (dali1.set_short_address, dali2.set_short_address):
                        os.rename(f1, f2)
                        self.clear_device(data['type'], data['address'])
                        self.update_device(data['type'], data['params'][0])
                    elif method in (dali1.swap_short_addresses, dali2.swap_short_addresses):
                        os.rename(f1, '/tmp/%s' % data['address'])
                        if os.path.exists(f2):
                            os.rename(f2, f1)
                        os.rename('/tmp/%s' % data['address'], f2)
                        self.update_device(data['type'], data['address'])
                        self.update_device(data['type'], data['params'][0])

        except BaseException as ex:
            logging.exception(ex)

    def send_progress(self, progress):
        async_to_sync(self.layer.group_send)(
            self.group,
            {
                'type': 'chat_message',
                'message': {
                    'progress': progress,
                }
            }
        )

    def send_status(self, channel_name):
        if os.path.exists(self.root):
            count = 0
            loading = 0

            dest = []
            for cd, dirs, files in os.walk(self.root):
                if len(files) > 0:
                    count += len(files)
                    dest.append((cd, files))

            for cd, files in dest:
                for address in sorted((json.loads(file)) for file in files):
                    loading += 1
                    async_to_sync(self.layer.send)(
                        channel_name,
                        {
                            'type': 'chat_message',
                            'message': {
                                'loading': loading * 100 // count,
                                'status': {
                                    'address': address,
                                    'type': cd.split(os.sep)[-1],
                                    'device': self.read(cd, address, default={})
                                }
                            }
                        }
                    )

        async_to_sync(self.layer.send)(channel_name, {'type': 'chat_message', 'message': {'loading': 100, 'status': False}})

==> resh-5.3.0.6-py3-none-any.whl: reface/line/scanner/scanner.py <==
import logging
import os
import threading

from asgiref.sync import async_to_sync

from reface.line.worker import Worker
from resh.domains import dali1, dali2
from resh.utils import const_values as v
from resh.utils.const import INSTANCE
from resh.utils.errors import DeviceNotAnswered
from resh.utils.settings import conf


class Scanner(Worker):
    folder = 'scan'
    marks = {}

    def file(self, domain, address_int): return os.path.join(self.root, domain, str(address_int))

    def _run_thread(self, data):
        addresses = data['addresses']
        keys = data['keys']

        if v.FADE_TIME in keys.get(dali1.Dali1Protocol.domain_name, []):
            keys[dali1.Dali1Protocol.domain_name].remove(v.FADE_TIME)
            if v.FADE_TIME_AND_RATE not in keys:
                keys[dali1.Dali1Protocol.domain_name].append(v.FADE_TIME_AND_RATE)

        if v.FADE_RATE in keys.get(dali1.Dali1Protocol.domain_name, []):
            keys[dali1.Dali1Protocol.domain_name].remove(v.FADE_RATE)
            if v.FADE_TIME_AND_RATE not in keys:
                keys[dali1.Dali1Protocol.domain_name].append(v.FADE_TIME_AND_RATE)

        d1 = addresses.get(v.DALI, [])
        d2 = addresses.get(v.DALI2, [])
        keys_1 = [v.TYPES]
        keys_2 = []
        for k in keys.get(v.COMMON, []):
            if k in dali1.dump.queries:
                keys_1.append(k)
            if k in dali2.dump.queries_device():
                keys_2.append(k)

        keys_1 += keys.get(dali1.Dali1Protocol.domain_name, []) + keys.get(v.SWITCH, []) + keys.get(v.COLOUR, [])
        keys_2 += keys.get(dali2.Dali2Protocol.domain_name, []) + keys.get(v.SENSORS, []) + keys.get(v.BUTTONS, [])

        total_count = len(d1) + len(d2)
        progress = 0

        self.send_progress(1)

        def on_step(*args, **kwargs):
            self.on_step(cmd.domain_name, *args, **kwargs)

        for a in sorted(set(d1+d2), key=lambda a: int(a[1:])):
            if self.stopped:
                break
            for d, keys, dump in ((d1, keys_1, dali1.dump), (d2, keys_2, dali2.dump)):
                if self.stopped:
                    break
                if a in d:
                    cmd = dump(self.module, self.channel, a)
                    conf.interface = self.box
                    cmd.set_step_handler(on_step)
                    cmd.custom_stack = keys
                    try:
                        cmd.exec()
                    except DeviceNotAnswered:
                        self.clear_device(cmd.domain_name, a)
                    except BaseException as ex:
                        print(str(ex))
                    progress += 1
                    self.send_progress(min(99, progress * 100 // total_count))

        self.send_progress(100)

    def stop(self):
        if self._command:
            self._command.set_break(None, None)
        self.send_progress(0)

    def on_step(self, dali_type,  address, key, value, index=None):
        if isinstance(value, DeviceNotAnswered):
            value = 'unanswered'
        elif isinstance(value, DeviceNotAnswered):
            value = 'can error'
        elif isinstance(value, BaseException):
            value = 'error'
        self.update_value(dali_type, address, key, value, index)

    def update_value(self, domain, address, key, value, index=None):
        data = self.read(domain, address, default={})

        if key == v.NUMBER_OF_INSTANCES:
            data['instances'] = data.get('instances', [])[0:value]
        elif index is not None:
            data.setdefault('instances', [])
            found = False
            for item in data['instances']:
                if item['index'] == index:
                    item[key] = value
                    found = True
                    break
            if not found:
                data['instances'].append({'index': index, key: value})
        else:
            if key == v.SCENES and data.get(key, None):
                data[key].update(value)
                value = data[key]
            else:
                data[key] = value

        self.write(domain, address, value=data)

        async_to_sync(self.layer.group_send)(
            self.group,
            {
                'type': 'chat_message',
                'message': {
                    'address': address,
                    'type': domain,
                    'index': index,
                    'data': {
                        key: value,
                    }
                }
            }
        )

    def clear_device(self, domain, address):
        """ Удаляет файл девайса и шлёт в сокет структуру для удаления

        :param domain: dali or dali2
        :param address: device address like "D0..D63" or "0..63"
        :return:
        """
        self.remove(domain, address[1:])

        async_to_sync(self.layer.group_send)(
            self.group,
            {
                'type': 'chat_message',
                'message': {
                    'remove': address,
                    'type': domain,
                }
            }
        )

    def update_device(self, domain, address):
        async_to_sync(self.layer.group_send)(
            self.group,
            {
                'type': 'chat_message',
                'message': {
                    'status': {
                        'address': address,
                        'type': domain,
                        'device': self.read(domain, address[1:], default={})
                    }
                }
            }
        )

    def set_value(self, action, data):
        type = data['type']
        key = data['key']
        value = data.get('value', '')
        args = [self.module, self.channel, data['address']]
        command = None

        if value == 'false':
            value = False
        elif value == 'true':
            value = True

        if type == dali1.dump.domain_name:
            if action == 'get' and key in (v.FADE_TIME, v.FADE_RATE):
                command = dali1.query_fade_time_and_rate
            else:
                command = dali1.dump.queries.get(key, {}).get(action)
                if command is None:
                    for commands_dict in dali1.dump.extended.values():
                        command = commands_dict.get(key, {}).get(action)
                        if command:
                            break
        elif type == dali2.dump.domain_name:
            command = dali2.dump.queries_device().get(key, {}).get(action)
            if command is None:
                args.append(data['instance'])
                command = dali2.dump.queries_instance().get(key, {}).get(action)
                if command is None:
                    if action == 'set' and key == v.IS_ENABLED:
                        if value:
                            command = dali2.enable_instance
                        else:
                            command = dali2.disable_instance
                    else:
                        for commands_dict in dali2.dump.queries_instance_type().values():
                            command = commands_dict.get(key, {}).get(action)
                            if command:
                                break

        conf.interface = self.box

        if command in (dali1.discovery, dali1.identify, dali2.identify):
            try:
                self.marks.get(command.domain_name, {}).get(key, {}).pop(data['address']).stop()
            except:
                pass

            if value:
                def __exec(inst):
                    _command = command(*args)
                    inst.marks.setdefault(command.domain_name, {})
                    inst.marks[command.domain_name].setdefault(key, {})
                    inst.marks[command.domain_name][key][data['address']] = command(*args)
                    inst.marks[command.domain_name][key][data['address']].exec()
                    inst.update_value(data['type'], data['address'][1:], key, False)

                threading.Thread(target=__exec, args=[self], name='qwe', daemon=False).start()

            if data['address'].startswith('D'):
                self.update_value(data['type'], data['address'][1:], key, value)

        elif command:
            if command.opt_params:
                args.append(data.get('value', ''))
            command = command(*args)
            res = command.exec()
            if res is not None:
                r_kwargs = {}
                if INSTANCE in command.def_params:
                    r_kwargs['index'] = command.instance._value
                self.update_value(type, command.address.address, key, res, **r_kwargs)

    def call_command(self, data):
        domain = dali1.Dali1Protocol if data['type'] == dali1.Dali1Protocol.domain_name else dali2.Dali2Protocol

        try:
            args = (self.module, self.channel, data['address'], *data['params'])
            method = domain.from_name(data['command'])
            conf.interface = self.box
            method(*args).exec()

            if method in (dali1.set_short_address, dali2.set_short_address,
                          dali1.swap_short_addresses, dali2.swap_short_addresses,
                          dali1.remove_short_address, dali2.remove_short_address):
                if method in (dali1.remove_short_address, dali2.remove_short_address):
                    self.clear_device(data['type'], data['address'])
                else:
                    f1 = self.file(data['type'], data['address'][1:])
                    f2 = self.file(data['type'], data['params'][0][1:])
                    if method in (dali1.set_short_address, dali2.set_short_address):
                        os.rename(f1, f2)
                        self.clear_device(data['type'], data['address'])
                        self.update_device(data['type'], data['params'][0])
                    elif method in (dali1.swap_short_addresses, dali2.swap_short_addresses):
                        os.rename(f1, '/tmp/%s' % data['address'])
                        if os.path.exists(f2):
                            os.rename(f2, f1)
                        os.rename('/tmp/%s' % data['address'], f2)
                        self.update_device(data['type'], data['address'])
                        self.update_device(data['type'], data['params'][0])

        except BaseException as ex:
            logging.exception(ex)

    def send_progress(self, progress):
        async_to_sync(self.layer.group_send)(
            self.group,
            {
                'type': 'chat_message',
                'message': {
                    'progress': progress,
                }
            }
        )

    def send_status(self, channel_name):
        # ToDo каким-то образом в папке '/tmp/rapida/1/1/3/scan/dali' появляется файл "U"

        if os.path.exists(self.root):
            count = 0
            loading = 0

            dest = []
            for cd, dirs, files in os.walk(self.root):
                if len(files) > 0:
                    count += len(files)
                    dest.append((cd, files))

            for cd, files in dest:
                for address in sorted(filter(lambda x: x.isdigit(), files), key=int):
                    loading += 1
                    async_to_sync(self.layer.send)(
                        channel_name,
                        {
                            'type': 'chat_message',
                            'message': {
                                'loading': loading * 100 // count,
                                'status': {
                                    'address': address,
                                    'type': cd.split(os.sep)[-1],
                                    'device': self.read(cd, address, default={})
                                }
                            }
                        }
                    )

        async_to_sync(self.layer.send)(channel_name, {'type': 'chat_message', 'message': {'loading': 100, 'status': False}})
